// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license.

// BLDVER.RC - Standard version resource for CoreXT applications.

// Before including this file into your .rc, define the following
// macros for your application (don't forget the \0's):
//     #define RC_VERSION_FILE_DESCRIPTION "Test DLL\0"
//     #define RC_VERSION_INTERNAL_NAME "Test\0"
//     #define RC_VERSION_ORIGINAL_FILE_NAME "Test.DLL\0"

#include <winver.h>
#include <bldver.h>

VS_VERSION_INFO VERSIONINFO

FILEVERSION INET_VER
PRODUCTVERSION INET_VER

FILEFLAGSMASK 0x3fL
#ifdef DEBUG
	FILEFLAGS 0x1L
#else
	FILEFLAGS 0x0L
#endif

//
// Global definitions
//
#ifndef RC_VERSION_COMPANY_NAME
    #define RC_VERSION_COMPANY_NAME "Microsoft Corporation\0"
#endif

#ifndef RC_VERSION_PRODUCT_NAME
    #define RC_VERSION_PRODUCT_NAME "Microsoft\256 Web Platform Extensions\0"
#endif

#ifndef RC_VERSION_LEGAL_COPYRIGHT
    #define RC_VERSION_LEGAL_COPYRIGHT "Copyright \251 Microsoft Corporation\0"
#endif

#ifndef RC_VERSION_LEGAL_TRADEMARKS
    #define RC_VERSION_LEGAL_TRADEMARKS "Microsoft\256 is a registered trademark of Microsoft Corporation.\0"
#endif

FILEOS 0x4L
FILETYPE 0x1L
FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "CompanyName", RC_VERSION_COMPANY_NAME
            VALUE "FileDescription", RC_VERSION_FILE_DESCRIPTION
            VALUE "InternalName", RC_VERSION_INTERNAL_NAME
            VALUE "LegalCopyright", RC_VERSION_LEGAL_COPYRIGHT
            VALUE "LegalTrademarks", RC_VERSION_LEGAL_TRADEMARKS
            VALUE "OriginalFilename", RC_VERSION_ORIGINAL_FILE_NAME
            VALUE "ProductName", RC_VERSION_PRODUCT_NAME
            VALUE "FileVersion", INET_VERSION
            VALUE "ProductVersion", INET_VERSION
            VALUE "PrivateBuild", BUILD_PRIVATE
#ifdef OLEREGISTER
            VALUE "OLESelfRegister","\0"
#endif
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
