// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

namespace JwtBearerSample;

public class Todo
{
    public string Description { get; set; }
    public string Owner { get; set; }
}
