// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

using Microsoft.AspNetCore.Server.IIS.FunctionalTests;
using Microsoft.AspNetCore.InternalTesting;
using Microsoft.Extensions.Logging.Testing;
using Xunit;

[assembly: CollectionBehavior(DisableTestParallelization = true)]
[assembly: RequiresIIS]
[assembly: ShortClassName]
