// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

namespace Microsoft.AspNetCore.JsonPatch.SystemTextJson;

public class NestedObject
{
    public string StringProperty { get; set; }
    public dynamic DynamicProperty { get; set; }
}
