// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.

#pragma once

#include "targetver.h"

#define WIN32_LEAN_AND_MEAN             // Exclude rarely-used stuff from Windows headers

#include <Windows.h>
#include <httpserv.h>
#include <cwchar>
#include <vector>
#include <shellapi.h>
#include <sstream>
#include <memory>
#include <filesystem>
#include <string>
#include <fstream>
