<div>
    <h3>
        Id_token content
    </h3>
    <p>{{userInfo.userName}}</p>
    <p>aud:{{userInfo.profile.aud}}</p>
    <p>iss:{{userInfo.profile.iss}}</p>
    <p>iat:{{userInfo.profile.iat}}</p>
    <p>nbf:{{userInfo.profile.nbf}}</p>
    <p>exp:{{userInfo.profile.exp}}</p>
    <p>ver:{{userInfo.profile.ver}}</p>
    <p>tid:{{userInfo.profile.tid}}</p>
    <p>amr:{{userInfo.profile.amr}}</p>
    <p>oid:{{userInfo.profile.oid}}</p>
    <p>upn:{{userInfo.profile.upn}}</p>
    <p>unique_name:{{userInfo.profile.unique_name}}</p>
    <p>sub:{{userInfo.profile.sub}}</p>
    <p>family_name:{{userInfo.profile.family_name}}</p>
    <p>given_name:{{userInfo.profile.given_name}}</p>
    <p>pwd_exp:{{userInfo.profile.pwd_exp}}</p>
    <p>pwd_url:{{userInfo.profile.pwd_url}}</p>

</div>