// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license.

HRESULT
CheckInstallToSharedConfig(
    IN  MSIHANDLE           hInstall,
        BOOL *              pbShouldInstall
    );
