// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

namespace Microsoft.AspNetCore.Http.Metadata;

/// <summary>
/// Defines a contract used to specify metadata for skipping the StatusCodePage
/// middleware in <see cref="Endpoint.Metadata"/>.
/// </summary>
public interface ISkipStatusCodePagesMetadata
{
}