// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

namespace CustomAuthorizationFailureResponse.Authentication;

public static class SampleAuthenticationSchemes
{
    public const string CustomScheme = "CustomScheme";
}
