// Licensed to the .NET Foundation under one or more agreements.
// The .NET Foundation licenses this file to you under the MIT license.

using System.Runtime.CompilerServices;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Formatters;

[assembly: TypeForwardedTo(typeof(InputFormatterException))]
[assembly: TypeForwardedTo(typeof(ProblemDetails))]
